@@ -105,36 +105,35 @@ int kill_wait(const poll_loop_args_t* args, pid_t pid, int sig)
             warn("process exited after %.1f seconds\n", secs);
             return 0;
         }
-        usleep(poll_ms * 1000);
+        struct timespec req = { .tv_sec = (time_t)(poll_ms / 1000), .tv_nsec = (poll_ms % 1000) * 1000000 };
+        nanosleep(&req, NULL);
     }
     errno = ETIME;
     return -1;
 }
 
 /*
- * Find the process with the largest oom_score and kill it.
+ * Find the process with the largest oom_score.
  */
-void kill_largest_process(const poll_loop_args_t* args, int sig)
+procinfo_t find_largest_process(const poll_loop_args_t* args)
 {
-    struct procinfo victim = { 0 };
-    struct timespec t0 = { 0 }, t1 = { 0 };
+    DIR* procdir = opendir("/proc");
+    if (procdir == NULL) {
+        fatal(5, "%s: could not open /proc: %s", __func__, strerror(errno));
+    }
 
+    struct timespec t0 = { 0 }, t1 = { 0 };
     if (enable_debug) {
         clock_gettime(CLOCK_MONOTONIC, &t0);
     }
 
-    DIR* procdir = opendir("/proc");
-    if (procdir == NULL) {
-        fatal(5, "Could not open /proc: %s", strerror(errno));
-    }
-
-    int candidates = 0;
+    procinfo_t victim = { 0 };
     while (1) {
         errno = 0;
         struct dirent* d = readdir(procdir);
         if (d == NULL) {
             if (errno != 0)
-                warn("userspace_kill: readdir error: %s", strerror(errno));
+                warn("%s: readdir error: %s", __func__, strerror(errno));
             break;
         }
 
@@ -143,7 +142,7 @@ void kill_largest_process(const poll_loop_args_t* args, int sig)
         if (!isnumeric(d->d_name))
             continue;
 
-        struct procinfo cur = {
+        procinfo_t cur = {
             .pid = (int)strtol(d->d_name, NULL, 10),
             .uid = -1,
             .badness = -1,
@@ -191,7 +190,6 @@ void kill_largest_process(const poll_loop_args_t* args, int sig)
         }
 
         debug(" badness %3d", cur.badness);
-        candidates++;
 
         if (cur.badness < victim.badness) {
             // skip "type 1", encoded as 1 space
@@ -220,6 +218,22 @@ void kill_largest_process(const poll_loop_args_t* args, int sig)
             continue;
         }
 
+        // Skip processes with oom_score_adj = -1000, like the
+        // kernel oom killer would.
+        int oom_score_adj = 0;
+        {
+            int res = get_oom_score_adj(cur.pid, &oom_score_adj);
+            if (res < 0) {
+                debug(" error reading oom_score_adj: %s\n", strerror(-res));
+                continue;
+            }
+            if (oom_score_adj == -1000) {
+                // skip "type 4", encoded as 3 spaces
+                debug("    \n");
+                continue;
+            }
+        }
+
         // Fill out remaining fields
         if (strlen(cur.name) == 0) {
             int res = get_comm(cur.pid, cur.name, sizeof(cur.name));
@@ -239,17 +253,33 @@ void kill_largest_process(const poll_loop_args_t* args, int sig)
 
         // Save new victim
         victim = cur;
-        debug(" uid %4d \"%s\" <--- new victim\n", victim.uid, victim.name);
+        debug(" uid %4d oom_score_adj %4d \"%s\" <--- new victim\n", victim.uid, oom_score_adj, victim.name);
 
     } // end of while(1) loop
     closedir(procdir);
 
-    if (candidates <= 1 && victim.pid == getpid()) {
-        warn("Only found myself (pid %d) in /proc. Do you use hidpid? See https://github.com/rfjakob/earlyoom/wiki/proc-hidepid\n",
-            victim.pid);
-        victim.pid = 0;
+    if (enable_debug) {
+        clock_gettime(CLOCK_MONOTONIC, &t1);
+        long delta = (t1.tv_sec - t0.tv_sec) * 1000000 + (t1.tv_nsec - t0.tv_nsec) / 1000;
+        debug("selecting victim took %ld.%03ld ms\n", delta / 1000, delta % 1000);
     }
 
+    if (victim.pid == getpid()) {
+        warn("%s: selected myself (pid %d). Do you use hidpid? See https://github.com/rfjakob/earlyoom/wiki/proc-hidepid\n",
+            __func__, victim.pid);
+        // zeroize victim struct
+        victim = (const procinfo_t) { 0 };
+    }
+
+    return victim;
+}
+
+/*
+ * Kill the victim process, wait for it to exit, send a gui notification
+ * (if enabled).
+ */
+void kill_process(const poll_loop_args_t* args, int sig, const procinfo_t victim)
+{
     if (victim.pid <= 0) {
         warn("Could not find a process to kill. Sleeping 1 second.\n");
         if (args->notify) {
@@ -259,12 +289,6 @@ void kill_largest_process(const poll_loop_args_t* args, int sig)
         return;
     }
 
-    if (enable_debug) {
-        clock_gettime(CLOCK_MONOTONIC, &t1);
-        long delta = (t1.tv_sec - t0.tv_sec) * 1000000 + (t1.tv_nsec - t0.tv_nsec) / 1000;
-        debug("selecting victim took %ld.%03ld ms\n", delta / 1000, delta % 1000);
-    }
-
     char* sig_name = "?";
     if (sig == SIGTERM) {
         sig_name = "SIGTERM";
